// ignore_for_file: avoid_print

library init;

import 'dart:io';
import 'package:process_run/shell.dart';

part 'init/files_and_folders.dart';
part 'init/files_templates.dart';
part 'init/enums.dart';

extension on String {
  /// this extension method will apply on [String]
  /// it will return a CamelCase from a given text
  String toCamelCase() {
    final camelCase = StringBuffer();
    split('_').forEach((namePart) => camelCase.write(
        '${namePart.trim()[0].toUpperCase()}${namePart.trim().substring(1)}'
            .trim()));

    return camelCase.toString();
  }
}

class Chaotic {
  /// it will add package to `pubspec.yaml` with latest version
  ///
  /// and run pub get command.
  /// ```dart
  /// addPackage(Packages.dio)
  /// ```
  static Future<void> addPackage(
    Packages package, [
    bool openDocumentation = false,
  ]) async {
    if (openDocumentation) {
      try {
        Shell().run('start https://pub.dev/packages/${_enumToString(package)}');
      } catch (e) {
        print(e.toString());
      }
    }
    try {
      await Shell().run('flutter pub add ${_enumToString(package)}');
    } catch (e) {
      print(e.toString());
    }
  }

  /// this method will trigger [addPackage] method multi time
  static Future<void> addPackages(List<Packages> packages,
      [bool openDocumentation = false]) async {
    for (final package in packages) {
      await addPackage(package, openDocumentation);
    }
  }

  /// make sure you deleted all files from lib folder before run this function.
  ///
  /// it will =>
  /// 1. create project structure.
  /// 2. add some required packages.
  /// 3. add assets to projects.
  /// 4. remove comments from pubspec.yaml .
  /// 5. run pub get command
  static Future<void> init([String initScreenName = 'splash']) async {
    await _createProjectStructure(initScreenName);
    await addPackages(_packagesTemp);
    await _initYaml();
    await _createNew('helper', _commonFolder, _helperTemp);
  }

  /// this method will called after [init] and [newScreen] method is called
  static Future<void> _generateRoutes(String screenName) async {
    final file = _getFile('router.dart', _serviceFolder);
    if (!file.existsSync()) {
      file.createSync(recursive: true);
      file.writeAsStringSync(_routerTemp(screenName));
    }
  }

  /// private method to create initial project structure
  static Future<void> _createProjectStructure(String initialScreenName) async {
    _arFile.createSync(recursive: true);
    await _arFile.writeAsString(_arTemp);
    _enFile.createSync(recursive: true);
    await _enFile.writeAsString(_enTemp);
    _mainFile.createSync(recursive: true);
    await _mainFile.writeAsString(_mainTemp);
    await Chaotic.newScreen(initialScreenName);
    await _analysisOptionsFile.writeAsString(_analysisOptionsTemp);
  }

  static Future<void> _initYaml() async {
    final yamlContent = await _yamlFile.readAsString();
    final newYamlContent = yamlContent.split('\n')
      ..removeWhere((line) => line.trim().startsWith('#'));

    await _yamlFile.writeAsString(newYamlContent.join('\n'));
    await _yamlFile.writeAsString(_assetsTemp, mode: FileMode.append);
  }

  /// this is a private method is used to create any new file
  static Future<void> _createNew(
    String fileName,
    Directory directory, [
    String template = '',
    File? exportFile,
  ]) async {
    try {
      final file = _getFile('$fileName.dart', directory);
      if (file.existsSync()) throw 'File Is Already Exists';
      file.createSync(recursive: true);
      file.writeAsStringSync('${template.trim()}\n');
      if (exportFile == null) return;
      if (directory.listSync().length <= 1) return;
      final exports = StringBuffer();
      for (final f in directory.listSync()) {
        exports.writeln('export \'${f.path.split('\\').last}\';');
      }
      exportFile.writeAsStringSync(exports.toString());
    } catch (e) {
      print(e.toString());
    }
  }

  /// this function will =>
  /// 1. create folder in screen folder
  /// 2. create file for screen class in this new folder
  /// 3. create file for provider class in this new folder
  /// 4. export this two files into screens.dart file
  static Future<void> newScreen(String screenName) async {
    final screenFolder = _getDir(screenName, _screenFolder);
    await Future.wait([
      // create screen
      _createNew(
        screenName,
        screenFolder,
        _screenTemp(screenName),
      ),
      // create provider
      _createNew(
        '${screenName}_provider',
        screenFolder,
        _providerTemp(screenName),
      ),
    ]);
    await _generateRoutes(screenName);
  }

  /// this function will =>
  /// 1. create file in common folder
  /// 2. export this file into commons.dart file
  static Future<void> newCommon(String commonName) async {
    return _createNew(commonName, _commonFolder);
  }

  /// this function will =>
  /// 1. create file widget class in widget folder
  /// 2. export this file into widgets.dart file
  static Future<void> newWidget(String widgetName) async {
    return _createNew(widgetName, _widgetFolder, _widgetTemp(widgetName));
  }

  /// this function will =>
  /// 1. create file model class in model folder
  /// 2. export this file into model.dart file
  static Future<void> newModel(String modelName) async {
    return _createNew(modelName, _modelFolder, _modelTemp(modelName));
  }

  /// this function will =>
  /// 1. create folder in assets/image
  /// 2. add this asset in pubspec.yaml and run pub get command
  static Future<void> newAsset(String newAsset) async {
    final newImageFolder = _getDir(newAsset, _imagesFolder);
    if (newImageFolder.existsSync()) {
      print('$newAsset is exists before');
      return;
    }
    newImageFolder.createSync(recursive: true);
    final yamlContent = await _yamlFile.readAsString();
    final newYamlContent = yamlContent.replaceAll('''assets:''', '''
assets:
    - assets/images/$newAsset/''');
    await _yamlFile.writeAsString(newYamlContent);
    await Shell().run('''flutter pub get''');
  }

  /// this function will =>
  /// 1. create file for this service in service folder
  /// 2. export this file into services.dart file
  static Future<void> newService(String serviceName) async {
    return _createNew(
      serviceName,
      _serviceFolder,
      _serviceTemp(serviceName),
    );
  }

  /// this function will =>
  /// 1. create file for this repository in repository folder
  /// 2. export this file into repositories.dart file
  static Future<void> newRepository(String repositoryName) async {
    return _createNew(
      repositoryName,
      _repositoryFolder,
      _repositoryTemp(repositoryName),
    );
  }

  /// this function will =>
  /// 1. create file for this provider in provider folder
  /// 2. export this file into providers.dart file
  static Future<void> newProvider(String provider) async {
    return _createNew(provider, _providerFolder, _providerTemp(provider));
  }

  /// this function will =>
  /// 1. create folder local_model in screen/[screenName] folder
  /// 2. create file in this folder
  /// 3. export this file into local_models.dart file
  static Future<void> newLocalModel(
    String modelName,
    String screenName,
  ) async {
    final screenFolder = _getDir(screenName, _screenFolder);
    final modelFolder = _getDir('model', screenFolder);
    final modelsFile = _getFile('models.dart', modelFolder);
    if (!screenFolder.existsSync()) await newScreen(screenName);
    return _createNew(
      modelName,
      modelFolder,
      _modelTemp(modelName),
      modelsFile,
    );
  }

  /// this function will =>
  /// 1. create folder local_widget in screen/[screenName] folder
  /// 2. create file in this folder
  /// 3. export this file into local_widgets.dart file
  static Future<void> newLocalWidget(
      String widgetName, String screenName) async {
    final screenFolder = _getDir(screenName, _screenFolder);
    final widgetFolder = _getDir('widget', screenFolder);
    final widgetsFile = _getFile('widgets.dart', widgetFolder);
    if (!screenFolder.existsSync()) await newScreen(screenName);
    return _createNew(
      widgetName,
      widgetFolder,
      _widgetTemp(widgetName),
      widgetsFile,
    );
  }

  /// call it at end of main function in bin/main.dart
  static Future<void> clear() async {
    File('./bin/main.dart').writeAsStringSync(_clearTemp());
  }
}
